#!/usr/bin/perl
#
# AUTHOR: [% author %], [% email %]
# CREATED: [% today %]

use [% min_perl_version %];

use strict;
use warnings;

our $VERSION = '0.01';

use Carp;
use POSIX qw(strftime);

__END__

=pod

=head1 NAME

[% module %] - <one line description>

=head1 VERSION

This documentation refers to [% module %] version 0.01.

=head1 USAGE

=head1 REQUIRED ARGUMENTS

=head1 OPTIONS

=head1 DESCRIPTION

=head1 REQUIREMENTS

None.

=head1 DIAGNOSTICS

None.

=head1 CONFIGURATION

None.

=head1 EXIT STATUS

None.

=head1 DEPENDENCIES

=over

=item * Carp

=item * POSIX

=back

=head1 INCOMPATIBILITIES

None reported.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any issues or feature requests to C<[% support_email %]>. Patches are welcome.

=head1 AUTHOR

[% author %] C<< [% email %] >>

=head1 COPYRIGHT AND LICENSE

Copyright (c) [% year %], [% author %] C<< [% email %] >>. All rights reserved.

[% license_body %]

=cut

